module tango_dom;

//import std.stdio;
import tango.io.Stdout;
import tango.text.xml.SaxParser;
import Float = tango.text.convert.Float;

import tools;


private class ContentHandler(Ch = char) : SaxHandler!(Ch) {
    public void startElement(Ch[] uri, Ch[] localName, Ch[] qName, Attribute!(Ch)[] atts) {
    }

    public void endElement(Ch[] uri, Ch[] localName, Ch[] qName) {
    }

    public void characters(Ch[] ch) {
//	Stdout.formatln(ch);
    }
}

private class CountHandler(Ch = char) : SaxHandler!(Ch) {
 public double res_total;
 double res_attr[MAX_LEVELS+1];
 double res_sum[MAX_LEVELS+1];
 int res_count[MAX_LEVELS+1];
 int level = 0;

 public void startDocument() {
    level = 0;
    res_total = 0;
    res_count[0] = 0;
 }
    
 public void startElement(Ch[] uri, Ch[] localName, Ch[] qName, Attribute!(Ch)[] atts) {
    int i;
    double res = 0;

    if (atts) {
	foreach (cur_attr; atts) {
	    auto value = cur_attr.value;
	    if (value) {
		try {
	            res = Float.toFloat(value);
		    if (res) break;
		} catch {
		}
	    }
	}
    }

//    printf("%i %lf\n", level, res);
    res_attr[level] = res;
    if (++level == MAX_LEVELS) {
	Stdout.formatln("XML structure is too deep, only {} levels are supported\n", MAX_LEVELS);
	throw new Exception("XML structure is too deep");
    }
    res_count[level]=0;
 }

 public void endElement(Ch[] uri, Ch[] localName, Ch[] qName) {
    if (res_count[level] > 3) {
	res_total += res_sum[level];
    }
    
    --level;
    if (res_count[level]) {
	res_sum[level]+=res_attr[level];
    } else {
	res_sum[level]=res_attr[level];
    }
    ++res_count[level];
 }

 public void characters(Ch[] ch) {
    if (ch) {
	try {
	    double res = Float.toFloat(ch);
	    if (res) res_attr[level-1] = res;
	} catch {
	}
    }

//	Stdout.formatln(ch);
 }
}


SaxParser!(char) parser = null;
SaxHandler!(char) handler = null;

extern(C) void initXML(TestData *td) {
    parser = new SaxParser!(char);

    if (get_walk_mode()) {
	handler = new CountHandler!(char);
    } else {
	handler = new ContentHandler!(char);
    }

    parser.setSaxHandler(handler);

}

extern(C) void releaseXML(TestData *td) {
}


extern(C) void parseXML(TestData *td, uint iter) {
    parser.setContent(td.xml[0..td.xmllen]);
    parser.parse();

    if (get_walk_mode()) {
//	Stdout.formatln("Sum {}", (cast(CountHandler!(char))handler).res_total);
    }
    
    parser.reset();
}

void main(char[][] args)
{
    DTest(args);
}
